package be.iminds.ilabt.jfed.espec.model;

import be.iminds.ilabt.jfed.espec.parser.ESpecDefaults;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * AnsiblePlaybookSpec
 */
public class AnsiblePlaybookSpec extends UploadLikeWithLogSpec {
    private final int debug;
    private final String extraVarsString;

    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    public AnsiblePlaybookSpec(@Nullable FileSource source,
                               @Nullable String path,
                               @Nullable String permissions,
                               @Nullable Integer debug,
                               @Nullable String log,
                               @Nullable String extraVars) {
        //nodes is special: always only on ansible node
        //                  value is ignored, but we put something anyway because null would mean ALL nodes and that's also not correct
        super(source, path, permissions == null ? "u=rwx" : permissions, Collections.singletonList("$ANSIBLE_NODE$"), log);

        this.debug = debug == null ? ESpecDefaults.ANSIBLE_DEBUG : debug;
        this.extraVarsString = extraVars;
    }

    public AnsiblePlaybookSpec(@Nullable FileSource source,
                               @Nullable String path,
                               @Nullable String permissions,
                               @Nullable Integer debug,
                               @Nullable String log,
                               @Nonnull Map<String, Object> extraVars) {
        //nodes is special: always only on ansible node
        //                  value is ignored, but we put something anyway because null would mean ALL nodes and that's also not correct
        super(source, path, permissions == null ? "u=rwx" : permissions, Collections.singletonList("$ANSIBLE_NODE$"), log);

        this.debug = debug == null ? ESpecDefaults.ANSIBLE_DEBUG : debug;
        try {
            this.extraVarsString = extraVars.isEmpty() ?
                    null :
                    MAPPER.writeValueAsString(extraVars);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert extra vars into string", e);
        }
    }


    public AnsiblePlaybookSpec(@Nullable FileSource source,
                               @Nullable String path,
                               @Nullable String permissions,
                               @Nullable Integer debug,
                               @Nullable String log) {
        this(source, path, permissions, debug, log, (String)null);
    }

    public boolean isDebug() {
        return debug > 0;
    }

    /**
     * @return number of v's to add to ansible arguments
     */
    public int getDebug() {
        return debug;
    }


    public boolean hasExtraVars() {
        return extraVarsString != null;
    }
    public String getExtraVarsString() {
        return extraVarsString;
    }
}
